#! /usr/bin/perl -w
use strict;
use Bio::SeqIO;
my $usage = "\n2fasta.pl Usage:\n
perl $0 [RBHB.out] [in_fasta] [db_fasta] [e_value] [out_dir]:\n
[RBHB.out]: The results of reciprocal best hits using BLASTP (RBHB)
[in_fasta]: The protein/nucleotide sequences of the genome of interest
[db_fasta]: The protein/nucleotide sequences of the reference genome
[e_value]: The blastp e-value cutoff
[out_dir]: The output directory fasta_dir_pr or fasta_dir_nt
$!\n";
my $rbhb = shift || die $usage;
my $in_fasta = shift || die $usage;
my $db_fasta = shift || die $usage;
my $evalue = shift || die $usage;
my $out_dir = shift || die $usage;
if (!(-d $out_dir)) {`mkdir $out_dir`;} else {`rm $out_dir/*`;}

my %hash;
my %hoh;
open RB, $rbhb || die "OOPS! Cannot open $rbhb for reading:$!\n";
while (<RB>) {
	chomp;
	my ($ccid,$geneid,$e) = (split /\t/)[0,1,2];
	next if ($e > $evalue);
	if ($geneid =~ /^jgi/) {$geneid = (split (/\|/,$geneid))[-2];}
	$hash{$ccid} ++;
	$hoh{$geneid}{$ccid} ++;
}
close RB;

my $seqio1 = Bio::SeqIO->new('-format'=>'fasta',-file=>$in_fasta);
while (my $seqobj = $seqio1->next_seq()) {
	my $seq = $seqobj->seq;
	my $ccid = $seqobj->id;
	next if (!(defined($hash{$ccid})));
	my $out = "$out_dir/$ccid.fasta";
	open OUT,'>',$out || die "OOPS! Cannot write to $out:$!\n";
	print OUT ">$ccid\n";
	print OUT "$seq\n";
	close OUT;
}

my $seqio2 = Bio::SeqIO->new('-format'=>'fasta',-file=>$db_fasta);
while (my $seqobj = $seqio2->next_seq()) {
	my $seq = $seqobj->seq;
	my $geneid = $seqobj->id;
	if ($geneid =~ /^jgi/) {$geneid = (split (/\|/,$geneid))[-2];}
	next if (!(defined($hoh{$geneid})));
	for my $ccid (keys %{$hoh{$geneid}}) {
		my $out = "$out_dir/$ccid.fasta";
		open OUT,'>>',$out||die "OOPS!Cannot write to $out:$!\n";
		print OUT ">$geneid\n";
		print OUT "$seq\n";
	}
}