#! /usr/bin/perl -w
use strict;
use Statistics::R;
my $usage = "\n$0 Usage:\n
perl $0 [in_file]:\n
[in_file]: The input file should follow the following format:
Gene_id	dNdS	Intensity_stage1	Intensity_stage2	Intensity_stage3	...
where
Column 1	=	The gene id
Column 2	=	The dN/dS ratio of that gene
Column 3--	=	The expression levels of that gene across developmental stages
$!\n";
my $in_file = shift || die $usage;

my %index;
my %total;
my $no_of_gene_tsi = 0;
my %dndsfn;
open IN,$in_file || die "OOPS! Cannot open $in_file:$!\n";
while (<IN>) {
	chomp;
	my ($ccid,$dnds,@array) = (split /\s+/);
	next if (!($dnds =~ m/\d$/));
	$no_of_gene_tsi ++;
	for my $i (0..$#array) {
		my $de = $array[$i] * $dnds;
		$index{$i} += $de;
		$total{$i} += $array[$i];
		$dndsfn{$no_of_gene_tsi}{$i} = $de;
	}
}
close IN;

print "Transcriptome Divergence Index (TDI):\t";
for my $i (sort {$a<=>$b} keys %index) {
	my $tsi = $index{$i} / $total{$i};
	print "\t".$tsi;
}
print "\n";

my $out_tmp = "data4stat.tab";
open OUT, '>', $out_tmp || die "OOPS! Cannot write to $out_tmp:$!\n";
for my $m (sort {$a<=>$b} keys %dndsfn) {
	print OUT $m;
	for my $i (sort {$a<=>$b} keys %index) {
		$dndsfn{$m}{$i} = $dndsfn{$m}{$i} / $total{$i} * $no_of_gene_tsi;
		print OUT "\t".$dndsfn{$m}{$i};
	}
	print OUT "\n";
}
close OUT;

my $R = Statistics::R->new();
$R->run(q`data<-read.table("data4stat.tab",sep="\t",header=FALSE,row.names=1)`);
print "Std. error:\t";
$R->run(q`b.mean <- function(data, num) {
	resamples <- lapply(1:num, function(i) sample(data, replace=T))
	r.mean <- sapply(resamples, mean)
	std.err <- sqrt(var(r.mean))
	list(std.err=std.err, resamples=resamples, means=r.mean)}`
);
for my $i (sort {$a<=>$b} keys %index) {
	$R->set('x', $i + 1);
	$R->run(q`b1<-b.mean(data[,x],1000);`);
	print "\t".$R->get('b1$std.err');
}
print "\n";
my $friedman_output = $R->run(q`friedman.test(as.matrix(data))`);
$friedman_output = (split (/\n/,$friedman_output))[-1];
print $friedman_output."\n";
$R->stop();
`rm $out_tmp`;