#! /usr/bin/perl -w
use strict;
use File::Basename;
my $usage = "\ncompute_dNdS.pl Usage:\n
perl $0 [fasta_dir_pr] [fasta_dir_nt]:\n
[fasta_dir_pr]: The directory of protein sequences of each gene pair
[fasta_dir_nt]: The directory of nucleotide sequences of each gene pair
$!\n";
my $fasta_dir_pr = shift || die $usage;
my $fasta_dir_nt = shift || die $usage;

my $out_dir_mafft = "output_mafft";
my $out_dir_pal2nal = "output_pal2nal";
my $out_dir_paml = "output_paml";
if (!(-d $out_dir_mafft)) {`mkdir $out_dir_mafft`;} else {`rm $out_dir_mafft/*`;}
if (!(-d $out_dir_pal2nal)) {`mkdir $out_dir_pal2nal`;} else {`rm $out_dir_pal2nal/*`;}
if (!(-d $out_dir_paml)) {`mkdir $out_dir_paml`;} else {`rm $out_dir_paml/*`;}

opendir (DIR,$fasta_dir_pr) || die "OOPS! Cannot open $fasta_dir_pr:$!\n";
my @files = grep {/fasta$/} readdir DIR;
close DIR;

foreach my $file (@files) {
	my $ccid = (split (/\./,$file))[0];
	my $pr_file = "$fasta_dir_pr/$file";
	my $nt_file = "$fasta_dir_nt/$ccid.fasta";
	my $out_file_mafft = "$out_dir_mafft/$ccid.mafft.out";
	my $out_file_pal2nal = "$out_dir_pal2nal/$ccid.paml";
	my $out_file_paml = "$out_dir_paml/$ccid.yn";
### Protein sequence alignments using MAFFT (L-INS-i option) ###
	`mafft-linsi $pr_file > $out_file_mafft`;
### Codon alignments using PAL2NAL ###
	`pal2nal.pl $out_file_mafft $nt_file -output paml > $out_file_pal2nal`;
### Computation of dN/dS ratios using PAML ###
	my $ctl = "yn00.ctl";
	open CTL,'>',$ctl || die "OOPS! Cannot write to $ctl:$!\n";
	print CTL "      seqfile = $out_file_pal2nal\n";
	print CTL "      outfile = $out_file_paml\n";
	print CTL "      verbose = 0\n        icode = 0\n    weighting = 0\n   commonf3x4 = 0\n*       ndata = 1\n";
	close CTL;
	`yn00`;
}
`rm yn00.ctl 2YN.dN 2YN.dS 2YN.t rst rst1 rub`;

### Extact results of the Yang & Nielsen (2000) method ###
opendir(YN,$out_dir_paml)||die"OOPS!Cannot open $out_dir_paml:$!\n";
@files = grep {/yn$/} readdir YN;
close YN;

my $out="dNdS_ratios.tab";
if(-f $out){`rm $out`;}
`touch $out`;
open OUT, '>' , $out || die "OOPS! Cannot write to $out:$!\n";
print OUT "Gene_id\tdNdS\tdN\tdS\n";
close OUT;

foreach my $file (@files) {
	my $yn_file = "$out_dir_paml/$file";
	my $ccid = (split (/\./,$file))[0];
	my $wc = `wc -l $yn_file`;
	$wc = (split(/\s+/,$wc))[0];
	next if ($wc<2);
	
	open OUT, '>>' ,$out || die"OOPS! Cannot write to $out:$!\n";
	print OUT $ccid."\t";
	my $info = `awk 'NR==91' $yn_file`;
	my ($dnds,$dn,$ds) = (split (/\s+/,$info))[-7,-6,-3];
	print OUT $dnds."\t".$dn."\t".$ds."\n";
	close OUT;
}