#! /usr/bin/perl -w
use strict;
my $usage = "\nBefore running this script, please read README.txt.
\nphylostratigraphy.pl Usage:\n
perl $0 [gene_list] [blastp_dir] [ref_genome_ps.tab] [taxid_lineage.txt] [blastp e-value cutoff]:\n
[gene_list]: The gene list of the genome of interest
[blastp_dir]: The path to the directory containing blastp results against reference genomes
[ref_genome_ps.tab]: The the reference genomes and their PS levels
[taxid_lineage.txt]: The .blastp.tab.taxid file generated by mapping_to_ncbi_taxonomy.pl in Step 2.2
[blastp e-value cutoff]: The selected blastp e-value cutoff
$!\n";
my $glist = shift || die $usage;
my $blastp_dir = shift || die $usage;
my $ref_genome_file = shift || die $usage;
my $lineage = shift || die $usage;
my $evalue = shift || die $usage;
my $ename = log($evalue)/log(0.1);

my $out = "all.ps.e$ename";

my %ids;
open GE,$glist || die "OOPS!Cannot open $glist for reading:$!\n";
while (<GE>) {
	chomp;
	my $id = (split /\s+/)[0];
	$ids{$id}++;
}
close GE;

my %orgs;
open REF,$ref_genome_file || die "OOPS!Cannot open $ref_genome_file for reading:$!\n";
while (<REF>) {
	chomp;
	my ($org,$ps) = (split /\s+/);
	$orgs{$ps}{$org}++;
}
close REF;

my %pss;
open LIN,$lineage || die "OOPS!Cannot open $lineage for reading:$!\n";
while (<LIN>) {
	chomp;
	my ($id,$e,$taxid,$name,$lineage) = (split /\t/);
	next if ($e > $evalue);
	my @array = (split (/\s/,$lineage));
	next if ($array[0] == 240176); #Exclude hit on the organism itself C. cinerea 240176
## PS1 ##
	next if ($array[-1] != 131567); #Exclude non-cellular organisms
	if (($array[-2] == 2157)||($array[-2] == 2)) { #Determine if in Archaea 2157 or Bacteria 2
		$pss{$id}{1} ++;
		next;
	}
## PS2 ##
	next if ($array[-2] != 2759); #Exclude non-eukaryotes
	if (($array[-3] == 33090) || ($array[-4] == 33208)) { #Determine if in Viridiplantae 33090 or Metazoa 33208
		$pss{$id}{2} ++;
		next;
	}
}
close LIN;

## PS3 to PS10##
for my $ps (sort {$a<=>$b} keys %orgs) {
	for my $org (keys %{$orgs{$ps}}) {
		my $file = "$blastp_dir/$org.blastp.tab";
		open IN,$file||die"OOPS!Cannot open $file for reading:$!\n";
		while (<IN>) {
			chomp;
			my ($id,$e) = (split /\t/)[0,-2];
			next if ($e > $evalue);
			next if (defined($pss{$id}));
			$pss{$id}{$ps} ++;
		}
		close IN;
	}
}

open OUT,'>',$out || die "OOPS!Cannot write to $out:$!\n";
for my $id (sort keys %ids) {
	my $ps = &min(keys %{$pss{$id}});
	if (defined($ps)) {
		print OUT $id."\t".$ps."\n";
	}else{
		print OUT $id."\t"."10"."\n";
	}
}
close OUT;

sub min {
	my ($min, @vars) = @_;
	for (@vars) {
		$min = $_ if $_ < $min;
	}
	return $min;
}