#! /usr/bin/perl -w
use strict;
use File::Basename;
my $usage = "\nmapping_to_ncbi_taxonomy.pl Usage:\n
perl $0 [in_blastp_tab] [gi_taxid_prot]:\n
[in_blastp_tab]: The output tab file of blastp against NCBI nr protein sequences
[gi_taxid_prot]: The gi_taxid_prot.dmp file downloaded from the NCBI taxonomy database
$!\n";
my $in_tab = shift || die $usage;
my $taxid_file = shift || die $usage;

my $basename = basename($in_tab);
my $out1 = "$basename.taxid";
my $out2 = "$basename.taxid.list";

my %GIs;
open GI, $in_tab || die "OOPS! Cannot open $in_tab for reading:$!\n";
while (<GI>) {
	my$gi = (split /\t/)[1];
	$gi = (split (/\|/,$gi))[1];
	$GIs{$gi} ++;
}
close GI;

my %taxid;
my %taxids;
open DATA, $taxid_file || die "OOPS! Cannot open $taxid_file for reading:$!\n";
while (<DATA>) {
	chomp;
    my($gi,$id) = (split /\t/);
    next if(!(defined($GIs{$gi})));
    $taxids{$id} ++;
    $taxid{$gi} = $id;
}
close DATA;

open OUT2, '>', $out2 || die "Cannot write to $out2:$!\n";
for my $tid(sort {$a<=>$b} keys %taxids) {
	print OUT2 $tid."\n";
}
close OUT2;

undef %taxids;

open OUT1, '>', $out1 || die "Cannot write to $out1:$!\n";
open FILE, $in_tab || die "OOPS!Cannot open $in_tab for reading:$!\n";
while (<FILE>) {
	chomp;
	my ($ccid,$ncbi,$evalue) = (split /\t/)[0,1,-2];
	$ccid = (split (/\s+/,$ccid))[0];
	my $gi = (split (/\|/,$ncbi))[1];
	next if (!($taxid{$gi}));
	print OUT1 $ccid."\t".$ncbi."\t".$evalue."\t".$taxid{$gi}."\n";
}
close FILE;
close OUT1;